#include <UniPWMControl.h>
#include <UniPWMMacros.h>

//#define ONBOARD_LED 13 //ersetzt durch TESTDIODE

// Pins for LEDs
#define RECEIVER_INP 2 // receiver input signal

#define POSLIGHT_R 3 // green constantly on
#define POSLIGHT_L 4 // red constantly on
#define REAR_LIGHT 5 // white constantly on
#define LANDING_LIGHT 6 // landinglights
#define MG 7 // MG simulation
#define BEACON_TOP 8 // slow red blink (Beacon)

#define LANDING_SERVO 9 // use pin 9 for analog servo 

#define WING_ACL 10 // white double flash, alternating with tail acl
#define BEACON_BOT 11 // slow red blink (Beacon)
#define TAIL_ACL 12 // white single flash, no driver, use external resistor
#define TESTDIODE 13 // Testdiode 13

// constant lights: on, off, dimmmed (max 255)
SEQUENCE( constOn ) = { HOLD( 255 ) };
SEQUENCE( constOff ) = { HOLD( 0 ) };
SEQUENCE( dimmedOn ) = { HOLD( 40 ) };

// single flash 200ms, dark 1200ms
SEQUENCE( singleFlash ) = { PAUSE(10), CONST(255,3), PAUSE(80) };

// double flash 2x200ms, pause 800ms
SEQUENCE( doubleFlash ) = { PAUSE(10), CONST(255,3), PAUSE(10), CONST(255,3), PAUSE(80) };

// MG with landing lights
SEQUENCE( mg ) = { PAUSE(10), CONST(255,3) };

// fader
SEQUENCE( fader ) = { PAUSE(20), RAMP(0,200,40), RAMP(200,0,40 ) };

// beacon - 2 ramps up - hold - 2 ramps down
SEQUENCE( beacon ) = { RAMP(0,45,20), RAMP(46,200,20), RAMP(200,45,20), RAMP(44,0,20), PAUSE(20) };

// beamer
SEQUENCE( beamOn ) = { CONST(0,40), RAMP(0,255,20 ), HOLD(250) };
SEQUENCE( beamOff ) = { RAMP(250,0,20), HOLD(0) };

// beamer retract servo 
#define RETRACT_POS 10 // 238
#define ASCEND_POS 255 // 198
SEQUENCE( retract ) = { RAMP( ASCEND_POS, RETRACT_POS, 100 ), HOLD( RETRACT_POS ) };
SEQUENCE( ascend ) = { RAMP( RETRACT_POS, ASCEND_POS, 100 ), HOLD( ASCEND_POS ) };
SEQUENCE( retracted) = { HOLD( RETRACT_POS ) };

UniPWMControl ctrl;

enum { PATTERN_STARTUP, PATTERN_FLIGHT, PATTERN_FLIGHT_MG, PATTERN_LOW, PATTERN_OFF, PATTERN_LANDING }; // Name new pattern here...

void setup()
{
Serial.begin(9600);

// high level object
ctrl.Init( 10 ); // max 10 output channels
ctrl.SetLowBatt( 510, PATTERN_OFF ); // Ausschalten (635 ~12,55 Volt, switch off value ~ 561 (3.7V bzw 11,1V gesamt) ueber Spannungsteiler + 8,2K Abgriff 2,7 K Masse an Pin A0
ctrl.SetInpChannelPin( RECEIVER_INP, UniPWMInpChannel::INP_NORMAL );

// retract servo  at startup
ctrl.AddSequence( PATTERN_STARTUP, LANDING_SERVO, ARRAY( retracted ), UniPWMChannel::ANALOG_SERVO ); // servo 

// flight 
ctrl.AddSequence( PATTERN_FLIGHT, TESTDIODE, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, BEACON_TOP, ARRAY( fader ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, BEACON_BOT, ARRAY( beacon ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, WING_ACL, ARRAY( doubleFlash ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, TAIL_ACL, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, POSLIGHT_L, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, POSLIGHT_R, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, REAR_LIGHT, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, MG, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_FLIGHT, LANDING_LIGHT, ARRAY( beamOff ) );
ctrl.AddSequence( PATTERN_FLIGHT, LANDING_SERVO, ARRAY( retract ), UniPWMChannel::ANALOG_SERVO );

// flightMG 
ctrl.AddSequence( PATTERN_FLIGHT_MG, TESTDIODE, ARRAY( doubleFlash ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, BEACON_TOP, ARRAY( fader ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, BEACON_BOT, ARRAY( beacon ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, WING_ACL, ARRAY( doubleFlash ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, TAIL_ACL, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, POSLIGHT_L, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, POSLIGHT_R, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, REAR_LIGHT, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, MG, ARRAY( mg ) ); 
ctrl.AddSequence( PATTERN_FLIGHT_MG, LANDING_LIGHT, ARRAY( beamOff ) );
ctrl.AddSequence( PATTERN_FLIGHT_MG, LANDING_SERVO, ARRAY( retract ), UniPWMChannel::ANALOG_SERVO );

// off swiched off lights if very low batt
ctrl.AddSequence( PATTERN_OFF, TESTDIODE, ARRAY( beacon ) ); 
ctrl.AddSequence( PATTERN_OFF, BEACON_TOP, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_OFF, BEACON_BOT, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_OFF, WING_ACL, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_OFF, TAIL_ACL, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_OFF, POSLIGHT_L, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_OFF, POSLIGHT_R, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_OFF, REAR_LIGHT, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_OFF, MG, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_OFF, LANDING_LIGHT, ARRAY( beamOff ) );
ctrl.AddSequence( PATTERN_OFF, LANDING_SERVO, ARRAY( retract ), UniPWMChannel::ANALOG_SERVO );

// Pattern_low  attention for low energy a little before pattern_Off is activated
ctrl.AddSequence( PATTERN_LOW, TESTDIODE, ARRAY( fader ) ); 
ctrl.AddSequence( PATTERN_LOW, BEACON_TOP, ARRAY( fader ) ); 
ctrl.AddSequence( PATTERN_LOW, BEACON_BOT, ARRAY( beacon ) ); 
ctrl.AddSequence( PATTERN_LOW, WING_ACL, ARRAY( doubleFlash ) ); 
ctrl.AddSequence( PATTERN_LOW, TAIL_ACL, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_LOW, POSLIGHT_L, ARRAY( dimmedOn ) ); 
ctrl.AddSequence( PATTERN_LOW, POSLIGHT_R, ARRAY( dimmedOn ) ); 
ctrl.AddSequence( PATTERN_LOW, REAR_LIGHT, ARRAY( dimmedOn ) ); 
ctrl.AddSequence( PATTERN_LOW, MG, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_LOW, LANDING_LIGHT, ARRAY( beamOff ) );
ctrl.AddSequence( PATTERN_LOW, LANDING_SERVO, ARRAY( retract ), UniPWMChannel::ANALOG_SERVO );

// landing
ctrl.AddSequence( PATTERN_LANDING, TESTDIODE, ARRAY( dimmedOn ) ); 
ctrl.AddSequence( PATTERN_LANDING, BEACON_TOP, ARRAY( fader ) ); 
ctrl.AddSequence( PATTERN_LANDING, BEACON_BOT, ARRAY( beacon ) ); 
ctrl.AddSequence( PATTERN_LANDING, WING_ACL, ARRAY( doubleFlash ) ); 
ctrl.AddSequence( PATTERN_LANDING, TAIL_ACL, ARRAY( singleFlash ) ); 
ctrl.AddSequence( PATTERN_LANDING, POSLIGHT_L, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_LANDING, POSLIGHT_R, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_LANDING, REAR_LIGHT, ARRAY( constOn ) ); 
ctrl.AddSequence( PATTERN_LANDING, MG, ARRAY( constOff ) ); 
ctrl.AddSequence( PATTERN_LANDING, LANDING_LIGHT, ARRAY( beamOn ) );
ctrl.AddSequence( PATTERN_LANDING, LANDING_SERVO, ARRAY( ascend ), UniPWMChannel::ANALOG_SERVO );

// assign 3 switch positions to corresponding patterns
ctrl.AddInputSwitchPos( 15, 24, PATTERN_LANDING );
ctrl.AddInputSwitchPos( 25, 31, PATTERN_FLIGHT_MG );
ctrl.AddInputSwitchPos( 30, 37, PATTERN_FLIGHT );
ctrl.AddInputSwitchPos( 38, 50, PATTERN_OFF );

// initalize patterns "flight" and "retracted servo "
ctrl.ActivatePattern( PATTERN_FLIGHT ); 
ctrl.ActivatePattern( PATTERN_STARTUP ); 
}


void loop()
{
  // battery warning at 11,8 volt
  int battVal = analogRead( 0x0 ); 
  if( battVal < 520 ) // Wert auf die "vorwarn"-Spannung anpassen (PATTERN_LOW)
    ctrl.ActivatePattern( PATTERN_LOW ); 
  else
    ctrl.DoLoop();

  delay( 500 );
}
